@@ -5,9 +5,10 @@ use strict;
 use warnings;
 use DBIx::Simple;
 use Params::Check;
+use List::Util qw(first);
 use Carp;
 
-our $VERSION = '0.51';
+our $VERSION = '0.52';
 $Params::Check::WARNINGS_FATAL = 1;
 $Params::Check::CALLER_DEPTH   = $Params::Check::CALLER_DEPTH + 1;
 
@@ -18,19 +19,21 @@ sub DEBUG { defined $_[1] ? ($DEBUG = $_[1]) : $DEBUG }
 
 #tablename
 sub TABLE {
-  croak("You must define a tablename for your class: sub TABLE {'tablename'}");
+  croak("You must define a tablename for your class: sub TABLE {'tablename'}!");
 }
 
 #table columns
 sub COLUMNS {
-  croak("You must define fields for your class: sub COLUMNS {['id','name','etc']}");
+  croak("You must define fields for your class: sub COLUMNS {['id','name','etc']}!");
 }
 
 #used to validate params to field-setters
 my $_CHECKS = {};
 
 sub CHECKS {
-  croak("You must define your CHECKS subroutine that returns your private \$_CHECKS");
+  croak(
+    "You must define your CHECKS subroutine that returns your private \$_CHECKS HASHREF!"
+  );
 }
 
 #default where
@@ -68,28 +71,34 @@ sub select {
     {%{$class->WHERE}, %$where})->object($class);
 }
 
+sub query {
+  my $class = shift;
+  return $class->dbix->query(@_)->object($class);
+}
+
 our $_attributes_made = {};
 
 sub _make_field_attrs {
   my $class = shift;
   (!ref $class)
-    || croak('Call this method as __PACKAGE__->make_field_attrs()');
+    || croak("Call this method as $class->make_field_attrs()");
   my $code = '';
-  foreach my $column (@{$class->COLUMNS()}) {
-    next if $class->can($column);    #careful: no redefine
+  foreach (@{$class->COLUMNS}) {
+    croak("You can not use '$_' as a column name since it is already defined in "
+        . __PACKAGE__ . '.')
+      if __PACKAGE__->can($_);
+    next if $class->can($_);    #careful: no redefine
     $code = "use strict;$/use warnings;$/use utf8;$/" unless $code;
-
-    #Carp::carp('Making sub ' . $column) if $DEBUG;
     $code .= <<"SUB";
-sub $class\::$column {
+sub $class\::$_ {
   my (\$self,\$value) = \@_;
   if(defined \$value){ #setting value
-    \$self->{data}{$column} = \$self->_check($column=>\$value);
+    \$self->{data}{$_} = \$self->_check($_=>\$value);
     #make it chainable
     return \$self;
   }
-  \$self->{data}{$column}
-    //= \$self->CHECKS->{$column}{default}; #getting value
+  \$self->{data}{$_}
+    //= \$self->CHECKS->{$_}{default}; #getting value
 }
 
 SUB
@@ -125,12 +134,15 @@ sub data {
   my ($self, $args) = _get_obj_args(@_);
   if (ref $args && keys %$args) {
     for my $field (keys %$args) {
-      unless (grep { $field eq $_ } @{$self->COLUMNS()}) {
+      unless (first { $field eq $_ } @{$self->COLUMNS()}) {
         Carp::cluck(
           "There is not such field $field in table " . $self->TABLE . '! Skipping...')
           if $DEBUG;
         next;
       }
+
+      #we may have getters/setters written by the author of the subclass
+      # so call each setter separately
       $self->$field($args->{$field});
     }
   }
@@ -188,7 +200,21 @@ sub insert {
 
 1;
 
-__END__
+
+# put a comment before __END__ marker so you can generate/use
+# additional perl tags using exuberant ctags.
+# example ctags filters to put in your ~/.ctags file:
+#--regex-perl=/^\s*?use\s+(\w+[\w\:]*?\w*?)/\1/u,use,uses/
+#--regex-perl=/^\s*?require\s+(\w+[\w\:]*?\w*?)/\1/r,require,requires/
+#--regex-perl=/^\s*?has\s+['"]?(\w+)['"]?/\1/a,attribute,attributes/
+#--regex-perl=/^\s*?\*(\w+)\s*?=/\1/a,aliase,aliases/
+#--regex-perl=/->helper\(\s?['"]?(\w+)['"]?/\1/h,helper,helpers/
+#--regex-perl=/^\s*?our\s*?[\$@%](\w+)/\1/o,our,ours/
+#--regex-perl=/^=head1\s+(.+)/\1/p,pod,Plain Old Documentation/
+#--regex-perl=/^=head2\s+(.+)/-- \1/p,pod,Plain Old Documentation/
+#--regex-perl=/^=head[3-5]\s+(.+)/---- \1/p,pod,Plain Old Documentation/
+
+#__END__
 
 =encoding utf8
 
@@ -202,11 +228,11 @@ This module is writen to replace most of the abstraction stuff from the base
 model class in the MYDLjE project on github, but can be used independently as well. 
 
 The class provides some useful methods which simplify representing rows from 
-tables as Perl objects. It is not intended to be a full featured ORM at all.
-It does not support relational mapping. This is left to the developer
-It is rather a database row abstraction. If you have to do complicated  SQL queries use directly 
-L<DBIx::Simple/query> method.
-
+tables as Perl objects and modifying them. It is not intended to be a full featured ORM 
+at all. It does not support relational mapping. This is left to the developer.
+It is rather a database table/row abstraction. 
+At the same time it is not just a fancy representation of a table row 
+like DBIx::Simple::Result::RowObject. See below for details.
 Last but not least, this module has no other non-CORE dependencies besides DBIx::Simple.
 
 =head1 SYNOPSIS
@@ -232,7 +258,7 @@ Last but not least, this module has no other non-CORE dependencies besides DBIx:
   sub CHECKS{$_CHECKS}
   1;#end of My::Model::AdminUser
 
-  #2. In as startup script or subroutine
+  #2. In a startup script or subroutine
   $app->{dbix} = DBIx::Simple->connect(...);
   #and/or
   DBIx::Simple::Class->dbix( $app->{dbix} );
@@ -240,15 +266,13 @@ Last but not least, this module has no other non-CORE dependencies besides DBIx:
   #3. usage 
   use My::Model::AdminUser;
   my $user = $dbix->select(
-    My::Model::AdminUser->TABLE,
-    '*',
-    {login_name => 'fred'}
+    My::Model::AdminUser->TABLE, '*', {login_name => 'fred'}
   )->object('My::Model::AdminUser')
   #or better (if SQL::Abstract is installed)
   my $user = My::Model::AdminUser->select(login_name => 'fred'); #this is cleaner
   
   $user->first_name('Fred')->last_name('Flintstone');#chainable setters
-  $user->save; #update user
+  $user->save; #update row
   #....
   my $user = My::Model::AdminUser->new(
     login_name => 'fred',
@@ -259,22 +283,20 @@ Last but not least, this module has no other non-CORE dependencies besides DBIx:
   print "new user has id:".$user->id;
   #...
   #select many
+  my $class = 'My::Model::AdminUser';
   my @admins = $dbix->select(
-    My::Model::AdminUser->TABLE,
-    My::Model::AdminUser->COLUMNS,
-    My::Model::AdminUser->WHERE
-  )->objects(My::Model::AdminUser);
-
+    $class->TABLE,
+    $class->COLUMNS,
+    $class->WHERE
+  )->objects($class);
+  #or
+  my @admins = $dbix->query(
+    $VERY_COMPLEX_SQL, @bind_variables
+  )->objects($class);
 
 
 =head1 CONSTANTS
 
-=head2 DEBUG
-
-Flag to enable debug warnings. Influencess all DBIx::Simple::Class subclasses.
-
-    DBIx::Simple::Class->DEBUG(1);
-
 =head2 TABLE
 
 You B<must> define it in your subclass. This is the table where 
@@ -343,47 +365,62 @@ to the current DBIx::Simple instance (with L<SQL::Abstract> support eventually :
   #or
   __PACKAGE__->dbix->query(...);#same instance
 
+=head2 DEBUG
+
+Flag to enable debug warnings. Influencess all DBIx::Simple::Class subclasses.
+
+    DBIx::Simple::Class->DEBUG(1);
 
 =head1 METHODS
 
 =head2 new
 
-The constructor.  
+Constructor.  
 Generates getters and setters (if needed) for the fields described in 
 L</COLUMNS>. Sets the eventually passed parameters as fields if they exists 
 as column names.
 
-  #Restore user object from sessiondata
-  if($self->sessiondata->{user_data}){
-    $self->user(My::User->new($self->sessiondata->{user_data}));
-  }
+My::User->new($session->{user_data});
 
 =head2 new_from_dbix_simple
 
 A constructor called in L<DBIx::Simple/object> and 
 L<DBIx::Simple/objects>. Basically makes the same as C<new()> without 
-checking the validity of the field values.
+checking the validity of the field values since they come from the 
+database and should be valid. See L<DBIx::Simple/Advanced_object_construction>.
 
   #This should be quicker than DBIx::Simple::Result::RowObject
+  my $class = 'My::Model::AdminUser';
   my @admins = $dbix->select(
-    My::Model::AdminUser->TABLE,
-    My::Model::AdminUser->COLUMNS,
-    My::Model::AdminUser->WHERE
-  )->objects(My::Model::AdminUser);
+    $class->TABLE,
+    $class->COLUMNS,
+    $class->WHERE
+  )->objects($class);
 
 =head2 select
 
-A convenient wrapper for L<DBIx::Simple/select> and constructor. 
+A convenient wrapper for 
+C<$dbix-E<gt>select($table,$columns,$where)-E<gt>object($class)> and constructor. 
 Note that L<SQL::Abstract> B<must be installed>. 
 Instantiates an object from a saved in the database row by constructing and 
 executing an SQL query based on the parameters. 
 These parameters are used to construct the C<WHERE> clause for the SQL C<SELECT> 
 statement. Prepends the L</WHERE> clause defined by you to the parameters. 
 If a row is found puts it in L</data>. 
-Returns C<$self>.
+Returns an instance of your class on success or C<undef> otherwise.
 
   my $user = MYDLjE::M::User->select(id => $user_id);
 
+=head2 query
+
+A convenient wrapper for C<$dbix-E<gt>query($SQL,@bind)-E<gt>object($class)> and constructor. 
+Accepts exacttly the same arguments as L<DBIx::Simple/query>.
+Returns an instance of your class on success or C<undef> otherwise.
+
+  my $user = My::User->query(
+    'SELECT ' . join (',',My::User->COLUMNS)
+    . ' FROM ' . My::User->TABLE.' WHERE id=? and disabled=?', 12345, 0);
+
 =head2 data
 
 Common getter/setter for all L</COLUMNS>. 
@@ -398,14 +435,14 @@ Returns a HASHREF - name/value pairs of the fields.
 =head2 save
 
 DWIM saver. If the object is fresh 
-( not instantiated via L</new_from_dbix_simple> ) prepares and 
+( not instantiated via L</new_from_dbix_simple> and L</select>) prepares and 
 executes an C<INSERT> statment, otherwise preforms an C<UPDATE>. 
 L</TABLE> and L</COLUMNS> are used to construct the SQL. 
 L</data> is stored as a row in L</TABLE>.
 
-    my $note = MyNote->new(title=>'My Title', description =>'This is a great story.');
-    #do something more...
-    $note->save;
+  my $note = MyNote->new(title=>'My Title', description =>'This is a great story.');
+  #do something more...
+  $note->save;
 
 =head2 insert
 
@@ -432,9 +469,18 @@ retreived from the database. Returns true on success.
 
 Красимир Беров, C<< <berov at cpan.org> >>
 
+=head1 CREDITS
+
+Jos Boumans for Params::Check
+
+Juerd Waalboer for DBIx::Simple
+
+Nate Wiger  and all contributors for SQL::Abstract
+
 =head1 BUGS
 
-Please report any bugs or feature requests to https://github.com/kberov/DBIx--Simple--Class/issues. 
+Please report any bugs or feature requests to 
+L<https://github.com/kberov/DBIx--Simple--Class/issues>. 
 I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
@@ -445,7 +491,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc DBIx::Simple::Class
 
-
 You can also look for information at:
 
 =over 4
@@ -471,7 +516,7 @@ L<http://search.cpan.org/dist/DBIx-Simple-Class/>
 
 =head1 SEE ALSO
 
-L<DBIx::Simple>, L<SQL::Abstract>, L<Params::Check>
+L<DBIx::Simple>, L<DBIx::Simple::Result::RowObject>, L<DBIx::Simple::OO> L<SQL::Abstract>, L<Params::Check>
 L<https://github.com/kberov/MYDLjE>
 
 
